#pragma once
#ifndef DATA_FEED_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
#define DATA_FEED_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketConnectionManager.h"

#include "JetByteTools/SocketTools/StreamSocketServerExCallback.h"

#include "JetByteTools/CoreTools/ThreadedCallbackTimerQueue.h"
#include "JetByteTools/CoreTools/LockableObject.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CConnectionCounter;

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

class CStreamSocketConnectionManager :
   public JetByteTools::Socket::CStreamSocketConnectionManager,
   private JetByteTools::Core::IQueueTimers::Timer,
   private JetByteTools::Socket::CStreamSocketServerExCallback
{
   public :

      CStreamSocketConnectionManager(
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         CConnectionCounter &connectionCounter,
         const JetByteTools::Milliseconds transferRateMillis,
         const long maxPendingWrites,
         const unsigned long messageSize);

      CStreamSocketConnectionManager(
         const CStreamSocketConnectionManager &rhs) = delete;

      ~CStreamSocketConnectionManager() override;

      CStreamSocketConnectionManager &operator=(
         const CStreamSocketConnectionManager &rhs) = delete;

      void DumpErrors() const;

   private :

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnOutgoingConnectionFailed(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         const DWORD lastError) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnWriteCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnWriteCompletionError(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         const DWORD lastError) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          const DWORD lastError) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnMaximumConnections() override;

      // Implement IQueueTimers::Timer

      void OnTimer(
         UserData userData) override;

      void SendData(
         JetByteTools::Socket::IStreamSocket &socket);

      void WriteIfPossible(
         JetByteTools::Socket::IStreamSocket &socket);

      void WriteData(
         JetByteTools::Socket::IStreamSocket &socket);

      CConnectionCounter &m_connectionCounter;

      const JetByteTools::Milliseconds m_transferRateMillis;

      long m_pendingWrites;

      const long m_maxPendingWrites;

      const unsigned long m_messageSize;

      BYTE *m_pMessageTemplate;

      unsigned long *m_pMessageSequence;

      unsigned long *m_pMessageType;

      JetByteTools::Core::CThreadedCallbackTimerQueue m_timerQueue;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_timerHandleIndex;

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      typedef std::map<DWORD, size_t> LastErrors;

      JetByteTools::Core::CLockableObject m_lock;

      LastErrors m_connectionErrors;
      LastErrors m_connectionResetErrors;
      LastErrors m_connectionErrorErrors;
      LastErrors m_writeCompletionErrors;

      StringErrors m_errors;

      void DumpErrors(
         const JetByteTools::Core::_tstring &errorType,
         const LastErrors &lastErrors) const;
};

#endif // DATA_FEED_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
