///////////////////////////////////////////////////////////////////////////////
// File: CommandLine.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "CommandLine.h"

#include "JetByteTools/CoreTools/Exception.h"
#include "JetByteTools/CoreTools/StringConverter.h"
#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"

#pragma hdrstop

#include <iostream>

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::CException;

using JetByteTools::Socket::CFlowControlListProcessor;

using std::string;
using std::cout;
using std::endl;

///////////////////////////////////////////////////////////////////////////////
// Constants
///////////////////////////////////////////////////////////////////////////////

static const _tstring s_maxBuffersQueuedArgument   = _T("maxBuffersQueued");
static const _tstring s_maxPendingWritesArgument   = _T("maxPendingWrites");
static const _tstring s_actionOnQueueFullArgument  = _T("actionOnQueueFull");

///////////////////////////////////////////////////////////////////////////////
// CCommandLine
///////////////////////////////////////////////////////////////////////////////

CCommandLineEx::CCommandLineEx(
   const DWORD argc,
   const char **argv,
   const _tstring &serverName)
   :  CCommandLine(
         argc,
         argv,
         serverName,
         CCommandLine::TCPServer |
         CCommandLine::SupportsPerfCounters |
         CCommandLine::SupportsSendRecvBufSize)
{
   m_parser.AddArgument(s_maxBuffersQueuedArgument);
   m_parser.AddArgument(s_maxPendingWritesArgument);
   m_parser.AddArgument(s_actionOnQueueFullArgument);
}

bool CCommandLineEx::Parse()
{
   bool ok = CCommandLine::Parse();

   if (ok)
   {
      _tstring message;

      ok = m_parser.Parse(m_argc, m_argv, message);

      if (ok)
      {
         if (m_parser.IsSet(s_actionOnQueueFullArgument))
         {
            const _tstring val = m_parser.GetValueAsString(s_actionOnQueueFullArgument);

            if (val != _T("f") &&
                val != _T("l") &&
                val != _T("t"))
            {
               message = _T("Invalid value for -actionOnQueueFull expected f, l or t");

               ok = false;
            }
         }
      }

      if (!ok)
      {
         ShowHelp(message);
      }
   }

   return ok ;
}

CCommandLineEx::Count CCommandLineEx::MaxBuffersQueued() const
{
   return StringToCount(m_parser.GetOptionalValueAsString(s_maxBuffersQueuedArgument, _T("10")).c_str());
}

CCommandLineEx::Count CCommandLineEx::MaxPendingWrites() const
{
   return StringToCount(m_parser.GetOptionalValueAsString(s_maxPendingWritesArgument, _T("1")).c_str());
}

CFlowControlListProcessor::ActionOnListFull CCommandLineEx::ActionOnBufferQueueFull() const
{
   const _tstring val = m_parser.GetOptionalValueAsString(s_actionOnQueueFullArgument, _T("t"));

   if (val == _T("f"))
   {
      return CFlowControlListProcessor::RemoveEarliest;
   }
   else if (val == _T("l"))
   {
      return CFlowControlListProcessor::RemoveLatest;
   }

   return CFlowControlListProcessor::TerminateConnection;
}

void CCommandLineEx::ShowMoreHelp()
{
   cout << " o -maxBuffersQueued   Defaults to 10." << endl;
   cout << " o -maxPendingWrites   Defaults to 1." << endl;
   cout << " o -actionOnQueueFull  f, l or t, defaults to t:" << endl;
   cout << "                       f = discard first buffer in queue" << endl;
   cout << "                       l = discard last buffer in queue" << endl;
   cout << "                       t = terminate connection." << endl;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: CommandLine.cpp
///////////////////////////////////////////////////////////////////////////////
