#pragma once
#ifndef DISTRIBUTION_SERVER_DISTRIBUTION_SERVER_INCLUDED__
#define DISTRIBUTION_SERVER_DISTRIBUTION_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: DistributionServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include "JetByteTools/SocketTools/FlowControlStreamSocketConnectionFilter.h"
#include "JetByteTools/SocketTools/FlowControlListProcessor.h"
#include "JetByteTools/SocketTools/NullStreamSocketFlowControlMonitor.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class IMonitorServerInstance;

namespace JetByteTools
{
   namespace Socket
   {
      class CStreamSocketBroadcastableConnectionCollection;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CDistributionServer
///////////////////////////////////////////////////////////////////////////////

class CDistributionServer :
   public JetByteTools::Socket::CFilteringStreamSocketServerEx,
   private CStreamSocketServerExCallback,
   private JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::IProcessBufferList,
   private JetByteTools::Socket::CNullStreamSocketFlowControlMonitor
{
   public :

      CDistributionServer(
         IMonitorServerInstance &monitor,
         JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::BufferCount maxBuffers,
         JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::PendingWriteCount maxPendingWrites,
         JetByteTools::Socket::CFlowControlListProcessor::ActionOnListFull action,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::SocketBufferSize recvBufferSize,
         JetByteTools::Socket::SocketBufferSize sendBufferSize,
         JetByteTools::Socket::ILimitConnections &connectionLimiter,
         JetByteTools::Socket::CStreamSocketBroadcastableConnectionCollection &distributor);

      CDistributionServer(
         const CDistributionServer &rhs) = delete;

      ~CDistributionServer() override;

      CDistributionServer &operator=(
         const CDistributionServer &rhs) = delete;

   private :

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionClientClose(
         JetByteTools::Socket::IStreamSocket &socket);

      void OnConnectionClosed(
         JetByteTools::Socket::IStreamSocket &socket) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnWriteCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      // Implement IProcessBufferList

      bool OnListFull(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IMonitorStreamSocketFlowControl::ConnectionId id,
         JetByteTools::Socket::IMonitorStreamSocketFlowControl &monitor,
         const JetByteTools::IO::IBuffer &newBuffer,
         JetByteTools::IO::IBufferChain &buffers,
         JetByteTools::IO::IBufferChain &discardedBuffers) override;

      void OnListStatus(
         JetByteTools::Socket::IStreamSocket &socket,
         const DWORD percentageUsed) override;

      void OnListEmpty(
         JetByteTools::Socket::IStreamSocket &socket) override;

      // Implement the bits of IMonitorStreamSocketFlowControl that we're interested in

      void OnBufferDelayed(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer) override;

      void OnBufferSent(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer) override;

      void OnBufferDiscarded(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer,
         const bool wasDelayed) override;

      void OnConnectionComplete(
         const ConnectionId id) override;

      void OnConnectionTerminated(
         const ConnectionId id) override;

      IMonitorServerInstance &m_monitor;

      JetByteTools::Socket::CStreamSocketBroadcastableConnectionCollection &m_distributor;

      JetByteTools::Socket::CFlowControlListProcessor m_listProcessor;

      JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter m_flowControlFilter;
};

#endif // DISTRIBUTION_SERVER_DISTRIBUTION_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: DistributionServer.h
///////////////////////////////////////////////////////////////////////////////
