#pragma once
#ifndef DISTRIBUTION_SERVER_FEED_SERVER_INCLUDED__
#define DISTRIBUTION_SERVER_FEED_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: FeedServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class IMonitorServerInstance;

namespace JetByteTools
{
   namespace Socket
   {
      class CStreamSocketBroadcastableConnectionCollection;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CFeedServer
///////////////////////////////////////////////////////////////////////////////

class CFeedServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback
{
   public :

      CFeedServer(
         IMonitorServerInstance &monitor,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::SocketBufferSize recvBufferSize,
         JetByteTools::Socket::SocketBufferSize sendBufferSize,
         JetByteTools::Socket::ILimitConnections &connectionLimiter,
         JetByteTools::Socket::CStreamSocketBroadcastableConnectionCollection &distributor);

      CFeedServer(
         const CFeedServer &rhs) = delete;

      ~CFeedServer() override;

      CFeedServer &operator=(
         const CFeedServer &rhs) = delete;

   private :

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionClosed(
         JetByteTools::Socket::IStreamSocket &socket) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void ProcessDataStream(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      IMonitorServerInstance &m_monitor;

      JetByteTools::Socket::CStreamSocketBroadcastableConnectionCollection &m_distributor;
};

#endif // DISTRIBUTION_SERVER_FEED_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: FeedServer.h
///////////////////////////////////////////////////////////////////////////////
