#pragma once
#ifndef ECHO_SERVER_PERFORMANCE_COUNTERS_INCLUDED__
#define ECHO_SERVER_PERFORMANCE_COUNTERS_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: PerformanceCounters.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/PerfMonTools/PerformanceMonitorCountersBase.h"

#include "PerformanceDataSchema.h"
#include "IMonitorServerInstance.h"

#include "JetByteTools/IOTools/IMonitorBufferAllocation.h"
#include "JetByteTools/IOTools/IMonitorIOPool.h"

#include "JetByteTools/SocketTools/IMonitorSocketAllocation.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Win32
   {
      class CSecurityAttributes;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CPerformanceCounters
///////////////////////////////////////////////////////////////////////////////

class CPerformanceCounters :
   public JetByteTools::IO::IMonitorBufferAllocation,
   public JetByteTools::IO::IMonitorIOPool,
   public JetByteTools::Socket::IMonitorSocketAllocation,
   private IMonitorServerInstance
{
   public:

      CPerformanceCounters(
         const bool createCounters,
         const JetByteTools::Core::_tstring &instanceName,
         JetByteTools::Win32::CSecurityAttributes &sa);

      CPerformanceCounters(
         const CPerformanceCounters &rhs) = delete;

      ~CPerformanceCounters();

      CPerformanceCounters &operator=(
         const CPerformanceCounters &rhs) = delete;

      static bool InstallCounters();

      static bool RemoveCounters();

      IMonitorServerInstance &GetServerInstanceMonitor(
         const JetByteTools::Core::_tstring &instanceName);

   private :

      // Implement IMonitorServerInstance

      void BytesProcessed(
         const long bytes) override;

      void OnConnectionStarted() override;

      void OnConnectionComplete() override;

      // Implement IMonitorBufferAllocation

      void OnBufferCreated() override;
      void OnBufferAllocated(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnBufferAllocationContention() override;
      void OnBufferReleaseContention() override;
      void OnBufferReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnBufferDestroyed() override;

      // Implement IMonitorIOPool

      void OnIOPoolError(
         const JetByteTools::Core::_tstring &message) override;
      void OnIOPoolThreadCreated() override;
      void OnIOPoolThreadStarted() override {}
      void OnIOPoolDispatch() override;
      void OnIOPoolWorkItemsDequeued(
         const DWORD /*numWorkItems*/) override {}
      void OnIOPoolThreadBeginProcessing() override;
      void OnIOPoolThreadEndProcessing() override;
      void OnIOPoolThreadStopped() override {}
      void OnIOPoolThreadDestroyed() override;

      // Implement IMonitorSocketAllocation

      void OnSocketCreated() override;
      void OnSocketAttached(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnSocketDestroyed() override;

      CPerformanceDataSchema m_schema;

      JetByteTools::PerfMon::CPerformanceMonitorCountersBase *m_pCounters;

      class CounterCollection
      {
         public :

            CounterCollection(
               JetByteTools::PerfMon::CPerformanceMonitorCountersBase *pCounters,
               const JetByteTools::Core::_tstring &instanceName,
               const bool instancePersists);

            ~CounterCollection();

            void ReleaseCounters();

            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::ObjectId m_object;

            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_bytesProcessedCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_bytesPerSecondCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_totalBuffersCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_buffersInUseCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_connectionsActiveCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_totalSocketsCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_socketsInUseCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_ioThreadsActiveCounter;
            const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_ioThreadsProcessingCounter;

      private :

            JetByteTools::PerfMon::CPerformanceMonitorCountersBase *m_pCounters;

            const bool m_instancePersists;
      };

      CounterCollection m_serverInstance;

      CounterCollection m_totalInstance;

      class ServerInstanceMonitor;

      typedef std::map<JetByteTools::Core::_tstring, ServerInstanceMonitor *> ServerInstanceMonitors;

      ServerInstanceMonitors m_serverInstanceMonitors;
};

#endif // ECHO_SERVER_PERFORMANCE_COUNTERS_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: PerformanceCounters.h
///////////////////////////////////////////////////////////////////////////////
