///////////////////////////////////////////////////////////////////////////////
// File: PerformanceDataSchema.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "PerformanceDataSchema.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;

using JetByteTools::PerfMon::DetailLevelNovice;
using JetByteTools::PerfMon::CounterTypeCounter32;
using JetByteTools::PerfMon::CounterTypeCounter64;
using JetByteTools::PerfMon::CounterTypePerSec32;

///////////////////////////////////////////////////////////////////////////////
// Static constants
///////////////////////////////////////////////////////////////////////////////

const _tstring CPerformanceDataSchema::s_serverExeObjectName = _T("JetByte Data Distribution Server - Exe");

const _tstring CPerformanceDataSchema::s_serverInstanceObjectName = _T("JetByte Data Distribution Server - Server Instance");

const _tstring CPerformanceDataSchema::s_bytesProcessedCounter = _T("Bytes");

const _tstring CPerformanceDataSchema::s_bytesPerSecondCounter = _T("Bytes/sec");

const _tstring CPerformanceDataSchema::s_totalBuffersCounter = _T("IO Buffers Total");

const _tstring CPerformanceDataSchema::s_buffersInUseCounter = _T("IO Buffers In Use");

const _tstring CPerformanceDataSchema::s_connectionsActiveCounter = _T("Connections Active");

const _tstring CPerformanceDataSchema::s_totalSocketsCounter = _T("Sockets Total");

const _tstring CPerformanceDataSchema::s_socketsInUseCounter = _T("Sockets In Use");

const _tstring CPerformanceDataSchema::s_ioThreadsActiveCounter = _T("IO Threads Active");

const _tstring CPerformanceDataSchema::s_ioThreadsProcessingCounter = _T("IO Threads Processing");

///////////////////////////////////////////////////////////////////////////////
// CPerformanceDataSchema
///////////////////////////////////////////////////////////////////////////////

CPerformanceDataSchema::CPerformanceDataSchema()
{
   Object &exeObject = AddObjectWithInstances(
      s_serverExeObjectName,
      _T("The \"JetByte Data Distribution Server - Exe\" object type defines ")
      _T("counters that apply to an instance of the the distribution server processes as ")
      _T("a whole. Each instance includes the process ID for that executable in ")
      _T("the name of the object. The _Total instance provides cumulative counts ")
      _T("for all currently active server processes."),
      DetailLevelNovice,
      5,                // 4 server exes plus _Total
      25);

   exeObject.AddCounter(
      s_bytesProcessedCounter,
      _T("The total number of bytes processed"),
      DetailLevelNovice,
      -6,
      CounterTypeCounter64);

   exeObject.SetDefaultCounter();

   exeObject.AddCounter(
      s_bytesPerSecondCounter,
      _T("The number of bytes processed per second"),
      DetailLevelNovice,
      -4,
      CounterTypePerSec32);

   exeObject.AddCounter(
      s_totalBuffersCounter,
      _T("The total number of IO buffers available"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_buffersInUseCounter,
      _T("The number of IO buffers currently in use"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_connectionsActiveCounter,
      _T("The number of connected sockets"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_totalSocketsCounter,
      _T("The total number of sockets in the socket pool"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_socketsInUseCounter,
      _T("The total number of sockets currently in use"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_ioThreadsActiveCounter,
      _T("The total number of I/O threads in the thread pool"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_ioThreadsProcessingCounter,
      _T("The number of I/O threads currently processing messages"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   Object &instanceObject = AddObjectWithInstances(
      s_serverInstanceObjectName,
      _T("The \"JetByte Data Distribution Server - Server Instance\" object type ")
      _T("defines counters that apply to an server listening on a port within a ")
      _T("server exe. There may be multiple servers listening on different ports ")
      _T("within a single executable. The parent object of this object is the ")
      _T("server executable object. ")
      _T("Each instance includes the port that the server is listening on in the ")
      _T("name of the object."),
      DetailLevelNovice,
      25,                     // each server listens on 5 ports, we allow 5 servers...
      25);

   instanceObject.AddCounter(
      s_bytesProcessedCounter,
      _T("The total number of bytes processed"),
      DetailLevelNovice,
      -6,
      CounterTypeCounter64);

   instanceObject.SetDefaultCounter();

   instanceObject.AddCounter(
      s_bytesPerSecondCounter,
      _T("The number of bytes processed per second"),
      DetailLevelNovice,
      -4,
      CounterTypePerSec32);

   instanceObject.AddCounter(
      s_connectionsActiveCounter,
      _T("The number of connected sockets"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: PerformanceDataSchema.cpp
///////////////////////////////////////////////////////////////////////////////
