///////////////////////////////////////////////////////////////////////////////
// File: ConnectionCounter.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "ConnectionCounter.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::IWaitable;

///////////////////////////////////////////////////////////////////////////////
// CConnectionCounter
///////////////////////////////////////////////////////////////////////////////

CConnectionCounter::CConnectionCounter(
   const long expectedConnections)
   :  m_expectedConnections(expectedConnections),
      m_established(0),
      m_failed(0)
{
}

CConnectionCounter::~CConnectionCounter()
{
}

void CConnectionCounter::ConnectionEstablished()
{
   ::InterlockedIncrement(&m_established);

   m_connectionsActive.Increment();

   m_expectedConnections.Decrement();
}

void CConnectionCounter::ConnectionFailed()
{
   ::InterlockedIncrement(&m_failed);

   m_expectedConnections.Decrement();
}

void CConnectionCounter::ConnectionComplete()
{
   m_connectionsActive.Decrement();
}

HANDLE CConnectionCounter::GetConnectionsCompleteWaitHandle() const
{
   return m_expectedConnections.GetAtZeroWaitObject().GetWaitHandle();
}

HANDLE CConnectionCounter::GetNoConnectionsActiveWaitHandle() const
{
   return m_connectionsActive.GetAtZeroWaitObject().GetWaitHandle();
}

long CConnectionCounter::ConnectionsEstablished() const
{
   return m_established;
}

long CConnectionCounter::ConnectionsFailed() const
{
   return m_failed;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: ConnectionCounter.cpp
///////////////////////////////////////////////////////////////////////////////
