#pragma once
#ifndef FEED_CLIENT_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
#define FEED_CLIENT_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketConnectionManager.h"

#include "JetByteTools/SocketTools/StreamSocketServerExCallback.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CConnectionCounter;

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

class CStreamSocketConnectionManager :
   public JetByteTools::Socket::CStreamSocketConnectionManager,
   private JetByteTools::Socket::CStreamSocketServerExCallback
{
   public :

      CStreamSocketConnectionManager(
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter,
         CConnectionCounter &connectionCounter);

      CStreamSocketConnectionManager(
         const CStreamSocketConnectionManager &rhs) = delete;

      ~CStreamSocketConnectionManager() override;

      CStreamSocketConnectionManager &operator=(
         const CStreamSocketConnectionManager &rhs) = delete;

      void DumpErrors() const;

   private :

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnOutgoingConnectionFailed(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         const DWORD lastError) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          const DWORD lastError) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnMaximumConnections() override;

      void ProcessData(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      CConnectionCounter &m_connectionCounter;

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      typedef std::map<DWORD, size_t> LastErrors;

      LastErrors m_connectionErrors;
      LastErrors m_connectionResetErrors;
      LastErrors m_connectionErrorErrors;

      StringErrors m_errors;

      void DumpErrors(
         const JetByteTools::Core::_tstring &errorType,
         const LastErrors &lastErrors) const;
};

#endif // FEED_CLIENT_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
