///////////////////////////////////////////////////////////////////////////////
// File: RoundTripTime.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "RoundTripTime.h"

#include "JetByteTools/CoreTools/Exception.h"
#include "JetByteTools/CoreTools/TickCountProvider.h"

#pragma hdrstop

#include <limits>

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::CException;
using JetByteTools::Core::CTickCountProvider;

///////////////////////////////////////////////////////////////////////////////
// CRoundTripTime
///////////////////////////////////////////////////////////////////////////////

CRoundTripTime::CRoundTripTime()
   :  m_sentTime(0),
      m_recvTime(0),
      m_status(Unsent)
{

}

CRoundTripTime::Status CRoundTripTime::GetStatus() const
{
   return m_status;
}

void CRoundTripTime::MarkAsSent()
{
   if (m_status != Unsent)
   {
      throw CException(_T("CRoundTripTime::MarkAsSent()"), _T("Incorrect status, expected 'Unsent'"));
   }

   m_sentTime = CTickCountProvider::TickCount();

   m_status = Sent;
}

void CRoundTripTime::MarkAsReceived()
{
   if (m_status != Sent)
   {
      throw CException(_T("CRoundTripTime::MarkAsReceived()"), _T("Incorrect status, expected 'Sent'"));
   }

   m_recvTime = CTickCountProvider::TickCount();

   m_status = Received;
}

DWORD CRoundTripTime::GetSentTime() const
{
   return m_sentTime;
}

DWORD CRoundTripTime::GetReceivedTime() const
{
   return m_recvTime;
}

DWORD CRoundTripTime::GetRoundTripTime() const
{
   if (m_status != Received)
   {
      return 0;
   }

   return (m_recvTime >= m_sentTime ? m_recvTime - m_sentTime : std::numeric_limits<DWORD>::max() - (m_sentTime - m_recvTime));
}

///////////////////////////////////////////////////////////////////////////////
// End of file: RoundTripTime.cpp
///////////////////////////////////////////////////////////////////////////////
