#pragma once
#ifndef SERVER_COMMON_STREAM_SOCKET_ALLOCATOR_INCLUDED__
#define SERVER_COMMON_STREAM_SOCKET_ALLOCATOR_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketAllocator.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketAllocator.h"
#include "JetByteTools/SocketTools/IMonitorSocketAllocation.h"

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketAllocator
///////////////////////////////////////////////////////////////////////////////

class CStreamSocketAllocator :
   public JetByteTools::Socket::CStreamSocketAllocator,
   private JetByteTools::Socket::IMonitorSocketAllocation
{
   public :

      CStreamSocketAllocator(
         SocketCount maxFreeSockets,
         JetByteTools::Socket::PerSocketBufferPoolingLimit maxPooledBuffers,
         JetByteTools::Socket::ConsecutiveSocketOperations maxConsecutiveOperations,
         bool log);

      CStreamSocketAllocator(
         JetByteTools::Socket::IMonitorSocketAllocation &monitor,
         SocketCount maxFreeSockets,
         JetByteTools::Socket::PerSocketBufferPoolingLimit maxPooledBuffers,
         JetByteTools::Socket::ConsecutiveSocketOperations maxConsecutiveOperations,
         bool log);

      CStreamSocketAllocator(
         const CStreamSocketAllocator &rhs) = delete;

      ~CStreamSocketAllocator();

      CStreamSocketAllocator &operator=(
         const CStreamSocketAllocator &rhs) = delete;

   private :

      void OnSocketCreated() override;

      void OnSocketAttached(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnSocketDestroyed() override;

      const bool m_log;
};

#endif // SERVER_COMMON_STREAM_SOCKET_ALLOCATOR_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketAllocator.h
///////////////////////////////////////////////////////////////////////////////
